/**
 * Copyright (C) 2015 TeeTime (http://teetime.sourceforge.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package teetime.stage;

import static org.hamcrest.collection.IsEmptyCollection.empty;
import static org.hamcrest.collection.IsIterableContainingInOrder.contains;
import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;
import static teetime.framework.test.StageTester.test;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

import teetime.util.ConstructorClosure;

/**
 * @author Nils Christian Ehmke
 */
public class ObjectProducerTest {

	@Test
	public void producerShouldSendSpecifiedNumberOfObjects() {
		final List<Integer> results = new ArrayList<Integer>();
		final ObjectProducer<Integer> producer = new ObjectProducer<Integer>(3, new Generator());

		test(producer).and().receive(results).from(producer.getOutputPort()).start();

		assertThat(results, contains(1, 2, 3));
	}

	public void producerShouldSendNothingIfSpecified() {
		final List<Integer> results = new ArrayList<Integer>();
		final ObjectProducer<Integer> producer = new ObjectProducer<Integer>(0, new Generator());

		test(producer).and().receive(results).from(producer.getOutputPort()).start();

		assertThat(results, is(empty()));
	}

	private static class Generator implements ConstructorClosure<Integer> {

		private int counter = 1;

		@Override
		public Integer create() {
			return new Integer(this.counter++);
		}
	}

}
